@@ -12,10 +12,9 @@ import subprocess
 import sys
 from logging import Logger
 from pathlib import Path
-from typing import Any, Dict
+from typing import Any, Dict, Optional
 
 from .. import log
-from ..exceptions import EnvironmentException
 from ..find_directories import (
     BINARY_NAME,
     CONFIGURATION_FILE,
@@ -29,6 +28,10 @@ from .command import CommandParser
 LOG: Logger = logging.getLogger(__name__)
 
 
+class InitializationException(Exception):
+    pass
+
+
 class Initialize(CommandParser):
     NAME = "initialize"
 
@@ -54,15 +57,29 @@ class Initialize(CommandParser):
         configuration: Dict[str, Any] = {}
 
         watchman_configuration_path = os.path.abspath(".watchmanconfig")
-        if shutil.which("watchman") is not None and log.get_yes_no_input(
-            "Also initialize a watchman configuration?"
+        watchman_path = shutil.which("watchman")
+        if watchman_path is not None and log.get_yes_no_input(
+            "Also initialize watchman in the current directory?"
         ):
             try:
-                with open(watchman_configuration_path, "w+") as configuration_file:
-                    configuration_file.write("{}\n")
-                subprocess.check_call(["watchman", "watch-project", "."])
-            except (IsADirectoryError, subprocess.CalledProcessError):
-                LOG.warning("Unable to initialize watchman for the current directory.")
+                if not os.path.isfile(watchman_configuration_path):
+                    with open(watchman_configuration_path, "w+") as configuration_file:
+                        configuration_file.write("{}\n")
+                    LOG.warning(
+                        f"Created basic `.watchmanconfig` at {watchman_configuration_path}"
+                    )
+                subprocess.run(
+                    [watchman_path, "watch-project", "."],
+                    check=True,
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    universal_newlines=True,
+                )
+                LOG.warning("Current directory is being watched by `watchman`.")
+            except IsADirectoryError:
+                LOG.warning(f"Unable to write to {watchman_configuration_path}.")
+            except subprocess.CalledProcessError:
+                LOG.warning("Failed to run `watchman watch-project .`.")
 
         binary_path = shutil.which(BINARY_NAME)
         if binary_path is None:
@@ -76,23 +93,23 @@ class Initialize(CommandParser):
                 )
             )
             if not os.path.isfile(binary_path):
-                raise EnvironmentException(
+                raise InitializationException(
                     "Unable to locate binary at `{}`.".format(binary_path)
                 )
         else:
             LOG.info("Binary found at `{}`".format(binary_path))
         configuration["binary"] = binary_path
 
-        typeshed = find_typeshed()
+        typeshed: Optional[Path] = find_typeshed()
         if typeshed is None:
-            typeshed = os.path.abspath(
+            typeshed = Path(
                 log.get_input("Unable to locate typeshed, please enter its root: ")
-            )
-            if not os.path.isdir(typeshed):
-                raise EnvironmentException(
+            ).resolve()
+            if not typeshed.is_dir():
+                raise InitializationException(
                     "No typeshed directory found at `{}`.".format(typeshed)
                 )
-        configuration["typeshed"] = typeshed
+        configuration["typeshed"] = str(typeshed)
 
         taint_models_path = find_taint_models_directory()
         if taint_models_path is not None:
@@ -127,38 +144,42 @@ class Initialize(CommandParser):
         return find_global_root(Path(".")) is not None
 
     def _run(self) -> None:
-        is_local = self._is_local()
-        current_directory = os.getcwd()
-        configuration_path = os.path.join(current_directory, CONFIGURATION_FILE)
-        if os.path.isfile(configuration_path):
-            if is_local:
-                error = (
-                    "Local configurations must be created in subdirectories of `{}`"
-                    "as it already contains a `.pyre_configuration`.".format(
-                        current_directory
+        try:
+            is_local = self._is_local()
+            current_directory = os.getcwd()
+            configuration_path = os.path.join(current_directory, CONFIGURATION_FILE)
+            if os.path.isfile(configuration_path):
+                if is_local:
+                    error = (
+                        "Local configurations must be created in subdirectories of `{}`"
+                        "as it already contains a `.pyre_configuration`.".format(
+                            current_directory
+                        )
+                    )
+                else:
+                    error = "A pyre configuration already exists at `{}`.".format(
+                        configuration_path
+                    )
+                raise InitializationException(error)
+            if os.path.isfile(configuration_path + ".local"):
+                raise InitializationException(
+                    "A local pyre configuration already exists at `{}`.".format(
+                        configuration_path + ".local"
                     )
                 )
+            if is_local:
+                configuration_path = configuration_path + ".local"
+                configuration = self._get_local_configuration()
             else:
-                error = "A pyre configuration already exists at `{}`.".format(
-                    configuration_path
-                )
-            raise EnvironmentException(error)
-        if os.path.isfile(configuration_path + ".local"):
-            raise EnvironmentException(
-                "A local pyre configuration already exists at `{}`.".format(
-                    configuration_path + ".local"
-                )
-            )
-        if is_local:
-            configuration_path = configuration_path + ".local"
-            configuration = self._get_local_configuration()
-        else:
-            configuration = self._get_configuration()
+                configuration = self._get_configuration()
 
-        with open(configuration_path, "w+") as configuration_file:
-            json.dump(configuration, configuration_file, sort_keys=True, indent=2)
-            configuration_file.write("\n")
-        LOG.info(
-            "Successfully initialized pyre! "
-            + "You can view the configuration at `{}`.".format(configuration_path)
-        )
+            with open(configuration_path, "w+") as configuration_file:
+                json.dump(configuration, configuration_file, sort_keys=True, indent=2)
+                configuration_file.write("\n")
+            LOG.log(
+                log.SUCCESS,
+                "Successfully initialized pyre! "
+                + "You can view the configuration at `{}`.".format(configuration_path),
+            )
+        except InitializationException as error:
+            LOG.error(f"{error}")
