@@ -5,6 +5,8 @@
 
 # pyre-unsafe
 
+import abc
+import dataclasses
 import glob
 import hashlib
 import json
@@ -15,9 +17,21 @@ import shutil
 import site
 import subprocess
 import sys
+from dataclasses import dataclass, field
 from logging import Logger
 from pathlib import Path
-from typing import Dict, List, Optional, Union
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from . import command_arguments, find_directories
 from .exceptions import EnvironmentException
@@ -35,6 +49,8 @@ from .resources import LOG_DIRECTORY
 
 LOG: Logger = logging.getLogger(__name__)
 
+T = TypeVar("T")
+
 
 def _relativize_root(root: str, project_root: str, relative_root: Optional[str]) -> str:
     if root.startswith("//"):
@@ -49,73 +65,112 @@ class InvalidConfiguration(Exception):
         super().__init__(self.message)
 
 
-class SearchPathElement:
-    def __init__(self, root: str, subdirectory: Optional[str]) -> None:
-        self.root = root
-        self.subdirectory = subdirectory
+class SearchPathElement(abc.ABC):
+    @abc.abstractmethod
+    def get_root(self) -> str:
+        raise NotImplementedError
 
-    @staticmethod
-    def expand(
-        path: Union[Dict[str, str], str],
-        project_root: str,
-        path_relative_to: Optional[str] = None,
+    @abc.abstractmethod
+    def path(self) -> str:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def command_line_argument(self) -> str:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def expand_root(
+        self, project_root: str, relative_root: Optional[str]
     ) -> "SearchPathElement":
-        if isinstance(path, str):
-            return SearchPathElement(
-                _relativize_root(path, project_root, path_relative_to),
-                subdirectory=None,
-            )
-        else:
-            if "root" in path and "subdirectory" in path:
-                root = path["root"]
-                subdirectory = path["subdirectory"]
-                return SearchPathElement(
-                    _relativize_root(root, project_root, path_relative_to),
-                    subdirectory=subdirectory,
-                )
-            elif "site-package" in path:
-                site_root = site.getsitepackages()
-                subdirectory = path["site-package"]
+        raise NotImplementedError
 
-                found_element = None
-                for root in site_root:
-                    site_package_element = SearchPathElement(
-                        _relativize_root(root, project_root, None),
-                        subdirectory=subdirectory,
-                    )
-                    if os.path.isdir(site_package_element.path()):
-                        found_element = site_package_element
-                if found_element is None:
-                    raise InvalidConfiguration(
-                        f"Cannot find site package '{subdirectory}'"
-                    )
-                return found_element
-            else:
-                raise InvalidConfiguration(
-                    "Search path elements must have `root` and `subdirectory` "
-                    "specified."
-                )
+
+@dataclasses.dataclass
+class SimpleSearchPathElement(SearchPathElement):
+    root: str
+
+    def get_root(self) -> str:
+        return self.root
 
     def path(self) -> str:
-        subdirectory = self.subdirectory
-        if subdirectory is not None:
-            return os.path.join(self.root, subdirectory)
-        else:
-            return self.root
+        return self.root
 
     def command_line_argument(self) -> str:
-        subdirectory = self.subdirectory
-        if subdirectory is not None:
-            return self.root + "$" + subdirectory
-        else:
-            return self.root
+        return self.root
 
-    def __eq__(self, other: str) -> bool:
-        # We support this for testing.
-        if isinstance(other, str):
-            return self.path() == other
-        else:
-            return self.root == other.root and self.subdirectory == other.subdirectory
+    def expand_root(
+        self, project_root: str, relative_root: Optional[str]
+    ) -> SearchPathElement:
+        return SimpleSearchPathElement(
+            _relativize_root(self.root, project_root, relative_root)
+        )
+
+
+@dataclasses.dataclass
+class SubdirectorySearchPathElement(SearchPathElement):
+    root: str
+    subdirectory: str
+
+    def get_root(self) -> str:
+        return self.root
+
+    def path(self) -> str:
+        return os.path.join(self.root, self.subdirectory)
+
+    def command_line_argument(self) -> str:
+        return self.root + "$" + self.subdirectory
+
+    def expand_root(
+        self, project_root: str, relative_root: Optional[str]
+    ) -> SearchPathElement:
+        return SubdirectorySearchPathElement(
+            root=_relativize_root(self.root, project_root, relative_root),
+            subdirectory=self.subdirectory,
+        )
+
+
+@dataclasses.dataclass
+class SitePackageSearchPathElement(SearchPathElement):
+    site_root: str
+    package_name: str
+
+    def get_root(self) -> str:
+        return self.site_root
+
+    def path(self) -> str:
+        return os.path.join(self.site_root, self.package_name)
+
+    def command_line_argument(self) -> str:
+        return self.site_root + "$" + self.package_name
+
+    def expand_root(
+        self, project_root: str, relative_root: Optional[str]
+    ) -> SearchPathElement:
+        # Site package does not participate in root expansion.
+        return self
+
+
+def create_search_paths(
+    json: Union[str, Dict[str, str]], site_roots: Iterable[str]
+) -> List[SearchPathElement]:
+    if isinstance(json, str):
+        return [SimpleSearchPathElement(json)]
+    elif isinstance(json, dict):
+        if "root" in json and "subdirectory" in json:
+            return [
+                SubdirectorySearchPathElement(
+                    root=json["root"], subdirectory=json["subdirectory"]
+                )
+            ]
+        elif "site-package" in json:
+            return [
+                SitePackageSearchPathElement(
+                    site_root=root, package_name=json["site-package"]
+                )
+                for root in site_roots
+            ]
+
+    raise InvalidConfiguration(f"Invalid search path element: {json}")
 
 
 def assert_readable_directory_in_configuration(
@@ -127,6 +182,304 @@ def assert_readable_directory_in_configuration(
         raise InvalidConfiguration(str(error))
 
 
+@dataclass(frozen=True)
+class PartialConfiguration:
+    autocomplete: Optional[bool] = None
+    binary: Optional[str] = None
+    buck_builder_binary: Optional[str] = None
+    disabled: Optional[bool] = None
+    do_not_ignore_all_errors_in: Sequence[str] = field(default_factory=list)
+    dot_pyre_directory: Optional[Path] = None
+    excludes: Sequence[str] = field(default_factory=list)
+    extensions: Sequence[str] = field(default_factory=list)
+    file_hash: Optional[str] = None
+    formatter: Optional[str] = None
+    ignore_all_errors: Sequence[str] = field(default_factory=list)
+    ignore_infer: Sequence[str] = field(default_factory=list)
+    logger: Optional[str] = None
+    number_of_workers: Optional[int] = None
+    other_critical_files: Sequence[str] = field(default_factory=list)
+    search_path: Sequence[SearchPathElement] = field(default_factory=list)
+    source_directories: Optional[Sequence[str]] = None
+    strict: Optional[bool] = None
+    taint_models_path: Sequence[str] = field(default_factory=list)
+    targets: Optional[Sequence[str]] = None
+    typeshed: Optional[str] = None
+    use_buck_builder: Optional[bool] = None
+    use_buck_source_database: Optional[bool] = None
+    version_hash: Optional[str] = None
+
+    @staticmethod
+    def _get_depreacted_map() -> Dict[str, str]:
+        return {"do_not_check": "ignore_all_errors"}
+
+    @staticmethod
+    def _get_extra_keys() -> Set[str]:
+        return {
+            "buck_mode",
+            "differential",
+            "stable_client",
+            "unstable_client",
+            "saved_state",
+            "taint_models_path",
+        }
+
+    @staticmethod
+    def from_command_arguments(
+        arguments: command_arguments.CommandArguments,
+    ) -> "PartialConfiguration":
+        return PartialConfiguration(
+            autocomplete=None,
+            binary=arguments.binary,
+            buck_builder_binary=arguments.buck_builder_binary,
+            disabled=None,
+            do_not_ignore_all_errors_in=[],
+            dot_pyre_directory=arguments.dot_pyre_directory,
+            excludes=arguments.exclude,
+            extensions=[],
+            file_hash=None,
+            formatter=arguments.formatter,
+            ignore_all_errors=[],
+            ignore_infer=[],
+            logger=arguments.logger,
+            number_of_workers=None,
+            other_critical_files=[],
+            search_path=[
+                SimpleSearchPathElement(element) for element in arguments.search_path
+            ],
+            source_directories=arguments.source_directories,
+            strict=arguments.strict,
+            taint_models_path=[],
+            targets=arguments.targets,
+            typeshed=arguments.typeshed,
+            use_buck_builder=arguments.use_buck_builder,
+            use_buck_source_database=arguments.use_buck_source_database,
+            version_hash=None,
+        )
+
+    @staticmethod
+    def from_string(contents: str) -> "PartialConfiguration":
+        def ensure_option_type(
+            json: Dict[str, Any], name: str, expected_type: Type[T]
+        ) -> Optional[T]:
+            result = json.pop(name, None)
+            if result is None:
+                return None
+            elif isinstance(result, expected_type):
+                return result
+            raise InvalidConfiguration(
+                f"Configuration `{name}` is expected to have type "
+                f"{expected_type} but got: `{json}`."
+            )
+
+        def is_list_of_string(elements: object) -> bool:
+            return isinstance(elements, list) and all(
+                isinstance(element, str) for element in elements
+            )
+
+        def ensure_optional_string_list(
+            json: Dict[str, Any], name: str
+        ) -> Optional[List[str]]:
+            result = json.pop(name, None)
+            if result is None:
+                return None
+            elif is_list_of_string(result):
+                return result
+            raise InvalidConfiguration(
+                f"Configuration `{name}` is expected to be a list of "
+                f"strings but got `{json}`."
+            )
+
+        def ensure_string_list(
+            json: Dict[str, Any], name: str, allow_single_string: bool = False
+        ) -> List[str]:
+            result = json.pop(name, [])
+            if allow_single_string and isinstance(result, str):
+                result = [result]
+            if is_list_of_string(result):
+                return result
+            raise InvalidConfiguration(
+                f"Configuration `{name}` is expected to be a list of "
+                f"strings but got `{json}`."
+            )
+
+        try:
+            configuration_json = json.loads(contents)
+
+            if configuration_json.pop("saved_state", None) is not None:
+                file_hash = hashlib.sha1(contents.encode("utf-8")).hexdigest()
+            else:
+                file_hash = None
+
+            dot_pyre_directory = ensure_option_type(
+                configuration_json, "dot_pyre_directory", str
+            )
+
+            search_path_json = configuration_json.pop("search_path", [])
+            if isinstance(search_path_json, list):
+                search_path = [
+                    element
+                    for json in search_path_json
+                    for element in create_search_paths(
+                        json, site_roots=site.getsitepackages()
+                    )
+                ]
+            else:
+                search_path = create_search_paths(
+                    search_path_json, site_roots=site.getsitepackages()
+                )
+
+            partial_configuration = PartialConfiguration(
+                autocomplete=ensure_option_type(
+                    configuration_json, "autocomplete", bool
+                ),
+                binary=ensure_option_type(configuration_json, "binary", str),
+                buck_builder_binary=ensure_option_type(
+                    configuration_json, "buck_builder_binary", str
+                ),
+                disabled=ensure_option_type(configuration_json, "disabled", bool),
+                do_not_ignore_all_errors_in=ensure_string_list(
+                    configuration_json, "do_not_ignore_all_errors_in"
+                ),
+                dot_pyre_directory=Path(dot_pyre_directory)
+                if dot_pyre_directory is not None
+                else None,
+                excludes=ensure_string_list(
+                    configuration_json, "excludes", allow_single_string=True
+                ),
+                extensions=ensure_string_list(configuration_json, "extensions"),
+                file_hash=file_hash,
+                formatter=ensure_option_type(configuration_json, "formatter", str),
+                ignore_all_errors=ensure_string_list(
+                    configuration_json, "ignore_all_errors"
+                ),
+                ignore_infer=ensure_string_list(configuration_json, "ignore_infer"),
+                logger=ensure_option_type(configuration_json, "logger", str),
+                number_of_workers=ensure_option_type(
+                    configuration_json, "number_of_workers", int
+                ),
+                other_critical_files=ensure_string_list(
+                    configuration_json, "critical_files"
+                ),
+                search_path=search_path,
+                source_directories=ensure_optional_string_list(
+                    configuration_json, "source_directories"
+                ),
+                strict=ensure_option_type(configuration_json, "strict", bool),
+                taint_models_path=ensure_string_list(
+                    configuration_json, "taint_models_path", allow_single_string=True
+                ),
+                targets=ensure_optional_string_list(configuration_json, "targets"),
+                typeshed=ensure_option_type(configuration_json, "typeshed", str),
+                use_buck_builder=ensure_option_type(
+                    configuration_json, "use_buck_builder", bool
+                ),
+                use_buck_source_database=ensure_option_type(
+                    configuration_json, "use_buck_source_database", bool
+                ),
+                version_hash=ensure_option_type(configuration_json, "version", str),
+            )
+
+            # Check for deprecated and unused keys
+            for (
+                deprecated_key,
+                replacement_key,
+            ) in PartialConfiguration._get_depreacted_map().items():
+                if deprecated_key in configuration_json:
+                    configuration_json.pop(deprecated_key)
+                    LOG.warning(
+                        f"Configuration file uses deprecated item `{deprecated_key}`. "
+                        f"Please migrate to its replacement `{replacement_key}`"
+                    )
+            extra_keys = PartialConfiguration._get_extra_keys()
+            for unrecognized_key in configuration_json:
+                if unrecognized_key not in extra_keys:
+                    LOG.warning(f"Unrecognized configuration item: {unrecognized_key}")
+
+            return partial_configuration
+        except json.JSONDecodeError as error:
+            raise InvalidConfiguration(f"Invalid JSON file: {error}")
+
+    @staticmethod
+    def from_file(path: Path) -> "PartialConfiguration":
+        try:
+            contents = path.read_text(encoding="utf-8")
+            return PartialConfiguration.from_string(contents)
+        except OSError as error:
+            raise InvalidConfiguration(f"Error when reading {path}: {error}")
+
+
+def merge_partial_configurations(
+    base: PartialConfiguration, override: PartialConfiguration
+) -> PartialConfiguration:
+    def overwrite_base(base: Optional[T], override: Optional[T]) -> Optional[T]:
+        return base if override is None else override
+
+    def append_base(base: Sequence[T], override: Sequence[T]) -> Sequence[T]:
+        return list(base) + list(override)
+
+    def raise_when_overridden(
+        base: Optional[T], override: Optional[T], name: str
+    ) -> Optional[T]:
+        if base is None:
+            return override
+        elif override is None:
+            return base
+        else:
+            raise InvalidConfiguration(
+                f"Configuration option `{name}` cannot be overridden."
+            )
+
+    return PartialConfiguration(
+        autocomplete=overwrite_base(base.autocomplete, override.autocomplete),
+        binary=overwrite_base(base.binary, override.binary),
+        buck_builder_binary=overwrite_base(
+            base.buck_builder_binary, override.buck_builder_binary
+        ),
+        disabled=overwrite_base(base.disabled, override.disabled),
+        do_not_ignore_all_errors_in=append_base(
+            base.do_not_ignore_all_errors_in, override.do_not_ignore_all_errors_in
+        ),
+        dot_pyre_directory=overwrite_base(
+            base.dot_pyre_directory, override.dot_pyre_directory
+        ),
+        excludes=append_base(base.excludes, override.excludes),
+        extensions=append_base(base.extensions, override.extensions),
+        file_hash=overwrite_base(base.file_hash, override.file_hash),
+        formatter=overwrite_base(base.formatter, override.formatter),
+        ignore_all_errors=append_base(
+            base.ignore_all_errors, override.ignore_all_errors
+        ),
+        ignore_infer=append_base(base.ignore_infer, override=override.ignore_infer),
+        logger=overwrite_base(base.logger, override.logger),
+        number_of_workers=overwrite_base(
+            base.number_of_workers, override.number_of_workers
+        ),
+        other_critical_files=append_base(
+            base.other_critical_files, override.other_critical_files
+        ),
+        search_path=append_base(base.search_path, override.search_path),
+        source_directories=raise_when_overridden(
+            base.source_directories,
+            override.source_directories,
+            name="source_directories",
+        ),
+        strict=overwrite_base(base.strict, override.strict),
+        taint_models_path=append_base(
+            base.taint_models_path, override.taint_models_path
+        ),
+        targets=raise_when_overridden(base.targets, override.targets, name="targets"),
+        typeshed=overwrite_base(base.typeshed, override.typeshed),
+        use_buck_builder=overwrite_base(
+            base.use_buck_builder, override.use_buck_builder
+        ),
+        use_buck_source_database=overwrite_base(
+            base.use_buck_source_database, override.use_buck_source_database
+        ),
+        version_hash=overwrite_base(base.version_hash, override.version_hash),
+    )
+
+
 class _ConfigurationFile:
     def __init__(self, file) -> None:
         self._deprecated = {"do_not_check": "ignore_all_errors"}
@@ -159,7 +512,7 @@ class _ConfigurationFile:
         if current:
             return current
         if value and print_on_success:
-            LOG.debug("Found %s: `%s`", key, ", ".join(value))
+            LOG.debug("Found %s: `%s`", key, ", ".join(str(value)))
         if value and key in self._deprecated:
             LOG.warning(
                 "Configuration file uses deprecated item `%s`: "
@@ -238,7 +591,9 @@ class Configuration:
         self._search_path: List[SearchPathElement] = []
         if search_path:
             search_path_elements = [
-                SearchPathElement.expand(path, project_root=project_root)
+                SimpleSearchPathElement(path).expand_root(
+                    project_root=project_root, relative_root=None
+                )
                 for path in search_path
             ]
             self._search_path.extend(search_path_elements)
@@ -389,12 +744,6 @@ class Configuration:
                 )
                 assert_readable_directory_in_configuration(typeshed_version_directory)
 
-        # Validate elements of the search path.
-        for element in self._search_path:
-            assert_readable_directory_in_configuration(
-                element.path(), field_name="search_path"
-            )
-
         if not is_list_of_strings(self.other_critical_files):
             raise InvalidConfiguration(
                 "`critical_files` field must be a list of strings."
@@ -435,16 +784,22 @@ class Configuration:
     def use_buck_source_database(self) -> bool:
         return self._use_buck_source_database or False
 
-    @property
-    def search_path(self) -> List[str]:
-        if not self._search_path:
-            return []
-        return [element.command_line_argument() for element in self._search_path]
-
     @property
     def local_root(self) -> Optional[str]:
         return self._local_root
 
+    def get_existent_search_paths(self) -> List[SearchPathElement]:
+        if not self._search_path:
+            return []
+        existent_paths = []
+        for search_path_element in self._search_path:
+            search_path = search_path_element.path()
+            if os.path.exists(search_path):
+                existent_paths.append(search_path_element)
+            else:
+                LOG.debug(f"Filtering out nonexistent search path: {search_path}")
+        return existent_paths
+
     def get_existent_do_not_ignore_errors_in_paths(self) -> List[str]:
         """
         This is a separate method because we want to check for existing files
@@ -520,7 +875,9 @@ class Configuration:
             parent_local_configuration = Configuration(
                 project_root=self.project_root,
                 local_root=str(parent_local_root),
-                search_path=[search_path.root for search_path in self._search_path],
+                search_path=[
+                    search_path.get_root() for search_path in self._search_path
+                ],
                 binary=self._binary,
                 typeshed=self._typeshed,
                 buck_builder_binary=self._buck_builder_binary,
@@ -631,24 +988,31 @@ class Configuration:
                 additional_search_path = configuration.consume(
                     "search_path", default=[]
                 )
+                site_roots = site.getsitepackages()
                 if isinstance(additional_search_path, list):
                     self._search_path.extend(
                         [
-                            SearchPathElement.expand(
-                                path,
+                            search_path_element.expand_root(
                                 project_root=self.project_root,
-                                path_relative_to=configuration_directory,
+                                relative_root=configuration_directory,
                             )
                             for path in additional_search_path
+                            for search_path_element in create_search_paths(
+                                path, site_roots=site_roots
+                            )
                         ]
                     )
                 else:
-                    self._search_path.append(
-                        SearchPathElement.expand(
-                            additional_search_path,
-                            project_root=self.project_root,
-                            path_relative_to=configuration_directory,
-                        )
+                    self._search_path.extend(
+                        [
+                            search_path_element.expand_root(
+                                project_root=self.project_root,
+                                relative_root=configuration_directory,
+                            )
+                            for search_path_element in create_search_paths(
+                                additional_search_path, site_roots=site_roots
+                            )
+                        ]
                     )
 
                 version_hash = configuration.consume(
